/*global document, window, console, jQuery*/
/*jslint maxlen: 150, this: true*/
/*jshint esversion: 6*/

// Author: John Sánchez Alvarez
// Create: 20160101

(function ($) {
    "use strict";

    /**
    * Retract sections
    **/
    $.extend({
        retract: function () {
            /* Private */
            var selector,
                class_i,
                class_t,
                class_c,
                class_ok,
                accordion,
                status_r = "none",
                time;

            /* Public */
            return {
                version: 20180521,

                /**
                 * Method for initializing retractable objects.
                 *
                 * @param {Object} attr_e Object with the initialization
                 *                        attributes of the retractable elements
                 * @returns {Object} Concatenator object
                 */
                init: function (attr_e) {
                    if (status_r !== "init") {
                        var attr = attr_e || {},
                            sentinel = false,
                            object_a;

                        selector = attr.selector || ".retract";
                        class_i = attr.class_i || "items";
                        class_t = attr.class_t || "title";
                        class_c = attr.class_c || "content";
                        class_ok = attr.class_ok || "active";
                        accordion = attr.accordion || false;
                        time = attr.time || 500;

                        //Toogle for all items
                        $(selector + " ." + class_i).each(function () {
                            if ($(this).data("open-retract") === undefined) {
                                $(this).find("." + class_c).hide();
                            } else {
                                $(this).find("." + class_c).show();
                                $(this).addClass(class_ok);
                            }

                            $(this).find("." + class_t).on("click", function () {
                                object_a = $(this).parent();

                                //Accordion
                                if (accordion) {
                                    sentinel = object_a.hasClass(class_ok);

                                    $(selector + " ." + class_i).removeClass(class_ok);
                                    $(selector + " ." + class_i + " ." + class_c).stop().hide(time);

                                    if (sentinel) {
                                        object_a.addClass(class_ok);
                                    }

                                    sentinel = false;
                                }

                                $(this).parent().toggleClass(class_ok);
                                $(this).siblings("." + class_c).stop().toggle(time);
                            });
                        });

                        status_r = "init";
                    }

                    return this;
                },

                /**
                 * Method to destroy the retractable objects.
                 *
                 * @returns {Object} Concatenator object
                 */
                destroy: function () {
                    if (status_r !== "destroy") {
                        $(selector + " ." + class_i).each(function () {
                            $(this).removeClass(class_ok);
                            $(this).find("." + class_t).off();
                            $(this).find("." + class_c).removeAttr("style");
                        });

                        status_r = "destroy";
                    }

                    return this;
                },

                /**
                 * Method for obtain status
                 *
                 * @returns {Strig} Status of retract
                 * */
                get_status: function () {
                    return status_r;
                }
            };
        }
    });
}(jQuery));
