<?php

namespace Drupal\tigo_portability\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\tigo_helpers\Controller\HelpersController;

/**
 *
 */
class PortabilityStatus extends ControllerBase {
  private $msisdn;
  private $dataSegment;

  /**
   *
   */
  public function __construct() {
    $this->msisdn = isset($_SESSION['msisdn_portability'])
            ? $_SESSION['msisdn_portability']
            : FALSE;
    $this->dataSegment = FALSE;
  }

  /**
   *
   */
  public function view() {

    if (isset($this->msisdn)) {
      $service = \Drupal::service('tigo_portability.portin_status');
      $data['msisdn'] = $this->msisdn;
      $service->msisdn($data);
      $response = $service->getResponse();
      if (isset($response->payload->status)) {
        switch ($response->payload->status) {
          case 'PIN_REQUEST_FORWARD_ERROR':
          case 'PORT_IN_ERROR':
          case 'PORT_CANCEL_SOLICITADO':
            $statusClass = 'status_error';
            $status = 'rechazada';
            $title = 'Lo sentimos, tu solicitud ha sido rechazada';
            $message = 'Verifica la información y el estado de tu línea actual e intenta de nuevo.';
            $this->data_segment = [
              'category' => 'Portabilidad',
              'event' => "PT - Estado NIP",
              'label' => 'Rechazada',
              'msisdn' => $this->msisdn,
            ];
            break;

          case 'PIN_REQUEST_ACEPTADO':
            $statusClass = 'status_proccess';
            $status = 'En proceso';
            $title = 'Finaliza tu proceso para pasarte a Tigo';
            $message = 'Estás a un solo paso para disfrutar de todos los beneficios que tenemos para ti. Ingresa tu código de verificación (NIP) para finalizarlo.';
            $url_data = [
              'msisdn' => $this->msisdn,
              'checked' => TRUE,
            ];
            $token_url = base64_encode(serialize($url_data));
            $this->data_segment = [
              'category' => 'Portabilidad - Recordar',
              'event' => "PT - Recordatorio",
              'label' => 'Continuar proceso',
              'msisdn' => $this->msisdn,
            ];
            break;

          case 'PORT_TIME_NOTIFICATION_CONFIRMADO':
          case 'PORT_IN_ACK_RECIBIDO':
            $statusClass = 'status_confirm';
            $status = 'Por confirmar';
            $title = 'Tu solicitud fue recibida y se encuentra en proceso de confirmación';
            $message = 'Recuerda que el paso de tu número tardará 3 días hábiles si es exitoso.';
            $last_message = '*Ten en cuenta que la fecha estimada de portación puede variar si la solicitud es aprobada o rechazada.';
            $last_message_2 = '**Cuando tu solicitud sea aceptada, cambia tu SIM actual por tu SIM Tigo para que empieces a disfrutar de tus servicios.';
            $portinDate = isset($response->payload->portingDate)
                            ? $response->payload->portingDate
                            : NULL;
            $this->dataSegment = [
              'category' => 'Portabilidad ',
              'event' => "PT - Recordatorio",
              'label' => 'Portabilidad exitosa en ' . $portinDate,
              'msisdn' => $this->msisdn,
            ];
            break;

          case 'LINEA_PORTADA':
            $statusClass = 'status_ok';
            $status = 'Activa';
            $title = '¡Bienvenido a Tigo!';
            $message = 'Ya eres parte de un mundo de beneficios diseñado exclusivamente para ti.';
            $last_message = 'Ten en cuenta que si aún no has cambiado tu SIM por la de Tigo, debes hacerlo para que cuentes con todos tus servicios.';
            $portinDate = isset($response->payload->portingDate)
                            ? $response->payload->portingDate
                            : NULL;
            $requestDate = isset($response->payload->requestDate)
                            ? $response->payload->requestDate
                            : NULL;
            $this->dataSegment = [
              'category' => 'Portabilidad ',
              'event' => "PT - Finaliza proceso",
              'label' => 'Proceso exitoso ' . $portinDate,
              'msisdn' => $this->msisdn,
            ];
            break;
        }

        $this->segment();

        $re = '/([\d]{3})([\d]{3})([\d]{4})/';
        $str = $this->msisdn;
        $subst = '($1) $2-$3';

        $result = preg_replace($re, $subst, $str);
        return [
          '#theme' => 'tigo-portability-status',
          '#status_class' => isset($statusClass)
          ? $statusClass
          : NULL,
          '#status' => isset($status)
          ? $status
          : NULL,
          '#msisdn' => isset($result)
          ? $result
          : NULL,
          '#number' => isset($this->msisdn)
          ? $this->msisdn
          : NULL,
          '#title' => isset($title)
          ? $title
          : NULL,
          '#message' => isset($message)
          ? $message
          : NULL,
          '#portin_date' => isset($portinDate)
          ? format_date(strtotime($portinDate), 'custom', 'd M Y')
          : NULL,
          '#request_date' => isset($requestDate)
          ? format_date(strtotime($requestDate), 'custom', 'd M Y')
          : NULL,
          '#last_message' => isset($last_message)
          ? $last_message
          : NULL,
          '#last_message_2' => isset($last_message_2)
          ? $last_message_2
          : NULL,
          '#token_url' => isset($token_url)
          ? $token_url
          : NULL,
          '#attached' => array(
            'library' => array(
              'tigo_portability/tigo_portability',
              'tigo/tigo.exists',
              'tigo/tigo.loader',
            ),
          ),
        ];
      }
    }

    if (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['SERVER_NAME']) {
      $base_url = $_SERVER['HTTP_X_FORWARDED_PROTO'] . '://' . $_SERVER['SERVER_NAME'];
    }
    else {
      global $base_url;
    }
    drupal_set_message('El número de línea que vas a pasar no se encuentra dentro de un proceso de solicitud. Por favor verífica e ingrésalo de nuevo.', 'error');
    header('Location: ' . $base_url . '/servicios/portabilidad');
    exit;
  }

  /**
   *
   */
  public function segment() {
    $helpers = new HelpersController();
    $segment = $helpers->segment_instance();
    $response = $segment->tracking_data_segment_generic(
        $this->msisdn,
        "PT - Portabilidad",
        $this->dataSegment
    );
  }

}
