<?php

namespace Drupal\tigo_portability\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\tigo_vtex_manage_orders\Services\Database\ConnectVtex;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\tigo_portability\Festivos;
use Drupal\tigo_vtex_api\VtexApi;
use Drupal\tigo_portability\Model\PortabilityModel;

/**
 * Controlador para ver el estado de la portacion.
 */
class PortabilityStatusVtex extends ControllerBase {
  /**
   * Variable para formatear conexion con la base de datos.
   *
   * @var \Drupal\tigo_vtex_manage_orders\Services\Database\ConnectVtex
   */
  protected $dbVtexOrders;
  /**
   * Variable para formatear fecha.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormat;
  /**
   * Variable para consultar servicios de vtex.
   *
   * @var \Drupal\tigo_vtex_api\VtexApi
   */
  protected $vtexApi;

  protected $portability;

  /**
   * Constructor del formulario.
   */
  public function __construct(
    ConnectVtex $db_vtex_orders,
    DateFormatter $dateFormat,
    VtexApi $vtex_api,
    PortabilityModel $portabilityModel) {
    $this->dbVtexOrders = $db_vtex_orders;
    $this->dateFormat = $dateFormat;
    $this->vtexApi = $vtex_api;
    $this->portability = $portabilityModel;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tigo_vtex.database'),
      $container->get('date.formatter'),
      $container->get('tigo_vtex_api.vtex_call'),
      $container->get('tigo_portability.portability_model')
    );
  }

  /**
   * Vista del controlador.
   */
  public function view(Request $request = NULL) {
    $order_id = $request->query->get('order_id');
    $item_id = $request->query->get('item_id');
    $new_ticket = $request->query->get('new_ticket');
    if ($new_ticket) {
      $status = 'status_confirm';
    }
    else {
      $status = 'status_proccess';
    }

    $result = $this->portability->getItemByItemId($item_id);

    if ($result && $result->ticket) {
      $festivo = new Festivos();
      $count = 0;
      $businessDays = 0;
      while ($businessDays < 3) {
        $count++;
        $nuevafecha = strtotime('+' . $count . ' day', strtotime($result->fecha_ticket));
        if (!$festivo->festivo(date('Y-m-d', $nuevafecha))) {
          $businessDays++;
        }
      }
      $jsonProduct = $this->vtexApi->getProduct($result->product_id);
      $productDescription = [];
      if ($jsonProduct) {
        $arrayProduct = json_decode($jsonProduct);
        $productName = $arrayProduct->Name;
        // $productDescription = $arrayProduct->Description;.
        $productDescription = explode("\r\n", $arrayProduct->Description);
      }

      $msisdn = $this->formatNumberPhone($result->linea_donante);

      $return = [
        '#theme' => 'tigo-portability-status-vtex',
        '#msisdn' => isset($msisdn)
        ? $msisdn
        : FALSE,
        '#sim_number' => $result->imsi,
        '#ticket' => $result->ticket,
        '#portin_request' => $this->dateFormat->format(strtotime($result->fecha_ticket), 'custom', 'd-m-Y'),
        '#portin_estimated' => $this->dateFormat->format($nuevafecha, 'custom', 'd-m-Y'),
        '#status_class' => $status,
        '#package_capacity' => isset($productName)
        ? $productName
        : FALSE,
        '#package_description' => isset($productDescription)
        ? $productDescription
        : FALSE,
        '#attached' => [
          'library' => [
            'tigo_portability/tigo_portability',
            'tigo/tigo.exists',
            'tigo/tigo.loader',
          ],
        ],
      ];
    }
    return isset($return)
     ? $return
     : [];
  }

  /**
   * Funcion para darle formato al msisdn.
   */
  public function formatNumberPhone($msisdn) {
    $re = '/([\d]{3})([\d]{3})([\d]{4})/';
    $str = $msisdn;
    $subst = '($1) $2-$3';
    return preg_replace($re, $subst, $str);
  }

}
