<?php

namespace Drupal\tigo_portability\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\tigo_portability\Services\Token;

/**
 * Clase que define el Token.
 *
 * @package Drupal\tigo_portability
 * @license https://transacciones.tigo.com.co/portabilidad/get-token  License 3.01
 * @link https://transacciones.tigo.com.co/portabilidad/get-token Generate new token
 */
class TokenController extends ControllerBase {

  /**
   * The Token service.
   *
   * @var \Drupal\tigo_portability\Services\Token
   */
  protected $token;

  /**
   * Constructor de la clase.
   */
  public function __construct(Token $token_service) {
    $this->token = $token_service;
  }

  /**
   * Servicios requeridos para Dependency Injection.
   *
   * @codeCoverageIgnore
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tigo_portability.token')
    );
  }

  /**
   * Solicita un nuevo Token valido.
   *
   * @return Symfony\Component\HttpFoundation\JsonResponse
   *   Con el nuevo token generado
   */
  public function getToken() {
    $access = $this->haveAccess();
    if ($access['code'] == 200) {
      $token = $this->token->getToken();
      $code = $token['code'];
      $message = $token['message'];
    }
    else {
      $token['token'] = FALSE;
      $code = $access['code'];
      $message = $access['message'];
    }
    return JsonResponse::create([
      'token' => $token['token'],
      'message' => $message,
    ], $code)->setSharedMaxAge(300);
  }

  /**
   * Verifica si la solicitud es de otro dominio.
   *
   * @return array
   *   Con la respuesta de validación del dominio.
   */
  public function haveAccess() {
    $return = [
      'code' => 200,
      'message' => 'Accesos concedidos',
    ];
    if (!isset($_SERVER['HTTP_REFERER']) ||
      $_SERVER['HTTP_REFERER'] === NULL ||
      $_SERVER['HTTP_REFERER'] == ''
    ) {
      $return = [
        'code' => 401,
        'message' => 'No tiene permisos',
      ];
    }
    return $return;
  }

}
