<?php

namespace Drupal\tigo_portability;

/**
 * Clase para calcular días habiles en Colombia.
 */
class Festivos {

  private $anioAct;
  private $arrFechas;

  /**
   * Constructor de la clase.
   */
  public function __construct() {
    $this->anioAct = date("Y");
    $pascua_actual = $this->pascua($this->anioAct);
    $this->arrFechas = [
      $this->anioAct . "-01-01",
      $this->getNextDay($this->anioAct . "-01-06"),
      $this->getNextDay($this->anioAct . "-03-19"),
      date("Y-m-d", strtotime("$pascua_actual -7 days")),
      date("Y-m-d", strtotime("$pascua_actual -3 days")),
      date("Y-m-d", strtotime("$pascua_actual -2 days")),
      $pascua_actual,
      $this->anioAct . "-05-01",
      date("Y-m-d", strtotime("$pascua_actual +43 days")),
      date("Y-m-d", strtotime("$pascua_actual +64 days")),
      date("Y-m-d", strtotime("$pascua_actual +71 days")),
      $this->getNextDay($this->anioAct . "-06-29"),
      $this->anioAct . "-07-20",
      $this->anioAct . "-08-07",
      $this->getNextDay($this->anioAct . "-08-15"),
      $this->getNextDay($this->anioAct . "-10-12"),
      $this->getNextDay($this->anioAct . "-11-01"),
      $this->getNextDay($this->anioAct . "-11-11"),
      $this->anioAct . "-12-08",
      $this->anioAct . "-12-25",
    ];
  }

  /**
   * Calculo festivos.
   */
  public function festivo($date_request) {
    $dc_dreq = date_create($date_request);
    $tday = date_format($dc_dreq, "w");
    if ($tday == 0) {
      return TRUE;
    }
    elseif ($tday > 0 && in_array($date_request, $this->arrFechas)) {
      return TRUE;
    }
    else {
      if (date("l", strtotime("$date_request")) == 'Saturday') {
        return TRUE;
      }
      else {
        return FALSE;
      }
    }
  }

  /**
   * Calculo fines de semana.
   */
  protected function getNextDay($datebase) {
    $dc_date = date_create($datebase);
    $nday = date_format($dc_date, "w");
    $nday = intval($nday);
    if ($nday == 0) {
      return date("Y-m-d", strtotime("$datebase +1 days"));
    }
    elseif ($nday == 1) {
      return $datebase;
    }
    elseif ($nday > 1) {
      $add_days = 8 - $nday;
      return date("Y-m-d", strtotime("$datebase +$add_days days"));
    }
  }

  /**
   * Semana santa.
   */
  protected function pascua($anno) {
    // Constantes mágicas.
    $m = 24;
    $n = 5;
    // Cálculo de residuos.
    $a = $anno % 19;
    $b = $anno % 4;
    $c = $anno % 7;
    $d = (19 * $a + $m) % 30;
    $e = (2 * $b + 4 * $c + 6 * $d + $n) % 7;
    // Decidir entre los 2 casos:
    if ($d + $e < 10) {
      $dia = $d + $e + 22;
      // Marzo.
      $mes = 3;
    }
    else {
      $dia = $d + $e - 9;
      // Abril.
      $mes = 4;
    }
    // Excepciones especiales (según artículo)
    // 4 = abril.
    if ($dia == 26  and $mes == 4) {
      $dia = 19;
    }
    // 4 = abril.
    if ($dia == 25 and $mes == 4 and $d == 28 and $e == 6 and $a > 10) {
      $dia = 18;
    }
    $ret = date("Y-m-d", strtotime($anno . '-' . $mes . '-' . $dia));
    return ($ret);
  }

}
