<?php

namespace Drupal\tigo_portability\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Manejo de la session php con el msisdn.
 *
 * @package Drupal\tigo_portabilidad
 * @license https://transacciones.tigo.com.co/servicios/paquetes  License 3.01
 * @link https://transacciones.tigo.com.co/servicios/portabilidad
 */
class AdminPortability extends ConfigFormBase {

  /**
   * Configuracion del formulario.
   *
   * @return array
   *   Devuelve la variable de configuración para editar
   */
  public function getEditableConfigNames() {
    return [
      'tigo_portability.adminsettings',
    ];
  }

  /**
   * Obtiene el formulario.
   *
   * @return string
   *   Devuelve el id del formulario
   */
  public function getFormId() {
    return 'admin_portability_form';
  }

  /**
   * Formulario.
   *
   * @param array $form
   *   Formulario.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Estado del Formulario.
   *
   * @return FormBase
   *   Devuelve la construcción del formulario
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('tigo_portability.adminsettings');

    $form['advanced'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('Configuración'),
    ];
    $form['general'] = [
      '#type' => 'details',
      '#title' => $this->t('Tigo Portabilidad'),
      '#group' => 'advanced',
    ];
    $form['general']['api'] = [
      '#type' => 'details',
      '#title' => $this->t('Portabilidad API'),
    ];
    $form['general']['api']['allowed_domains'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Dominios autorizados'),
      '#description' => $this->t('Registra los dominios que tienen permisos para consumir los servicios de portación'),
      '#default_value' => $config->get('allowed_domains'),
    ];
    $form['general']['token'] = [
      '#type' => 'details',
      '#title' => $this->t('Portabilidad Token'),
    ];
    $form['general']['token']['time_available'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Tiempo activo'),
      '#description' => $this->t('Tiempo de vida del token en minutos'),
      '#default_value' => $config->get('time_available'),
    ];
    $form['general']['gestor'] = [
      '#type' => 'details',
      '#title' => $this->t('Portabilidad Gestor'),
    ];
    $form['general']['gestor']['authorization'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Autorización'),
      '#description' => $this->t('Autorización para el servicio de gestores'),
      '#default_value' => $config->get('authorization'),
    ];
    $form['general']['gestor']['platform_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Platform ID'),
      '#description' => $this->t('Package id para el servicio de gestores'),
      '#default_value' => $config->get('platform_id'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * Valida los campos del formulario.
   *
   * @param array $form
   *   Formulario.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Estado del Formulario.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $this->config('tigo_portability.adminsettings')
      ->set('allowed_domains', $form_state->getValue('allowed_domains'))
      ->set('time_available', $form_state->getValue('time_available'))
      ->set('authorization', $form_state->getValue('authorization'))
      ->set('platform_id', $form_state->getValue('platform_id'))
      ->save();
  }

}
