<?php

namespace Drupal\tigo_portability\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\tigo\Model\SlideToUnlock;
use Drupal\tigo_portability\Services\ValidationVtex;
use Drupal\tigo_portability\Model\PortabilityModel;
use Drupal\tigo_portability\PortPospago;
use Drupal\tigo_portability\PortPrepago;
use Drupal\tigo_activation\Activation;
use Drupal\tigo_helpers\Controller\HelpersController;

/**
 * Formulario para portabilidad.
 */
class Portability extends FormBase {
  /**
   * Variable para formatear conexion con la base de datos.
   *
   * @var \Drupal\tigo_portability\Services\ValidationVtex
   */
  protected $validationVtex;
  protected $portability;
  protected $drupal;

  /**
   * Constructor del formulario.
   */
  public function __construct(
    ValidationVtex $validation_vtex,
    PortabilityModel $portabilidad,
    $drupal = NULL
  ) {
    $this->validationVtex = $validation_vtex;
    $this->portability = $portabilidad;
    $this->drupal = $drupal;
    if (!is_object($this->drupal)) {
      $this->drupal = new \Drupal();
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tigo_portability.validation_vtex'),
      $container->get('tigo_portability.portability_model')
    );
  }

  /**
   * Id del formualario.
   */
  public function getFormId() {
    return 'tigo_portability';
  }

  /**
   * Formulario consulta y portabilidad.
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
    Request $request = NULL
  ) {
    $email = $request->query->get('email');
    $donorline = isset($_SESSION['donor_line']) ? $_SESSION['donor_line'] : '';
    $port_type = isset($_SESSION['port_type']) ? $_SESSION['port_type'] : '';
    $exist_el = !empty($email) && !empty($donorline);
    $class = '';
    $markupModal = '<div class="modalsign__close"></div>
      <h2>Firma tu contrato digital</h2>
      <p>Para continuar con tu proceso.</p>
      <ul class="list-description">
        <li>Tómate una fotografía.</li>
        <li>Tómale una foto a tú cédula original.</li>
        <li>Haz tú firma figital.</li>
        <li>Acepta el págare.</li>
      </ul>
      <small>
        <p>*Por tú seguridad validaremos tus datos personales. La información que brindes debe ser veraz.</p>
        <p>*Iniciarás una solicitud que será procesada en máximo 24 horas y tardará 3 días hábiles si es exitosa.</p>
        <p>*Si tu paso de número a Tigo es exitoso, enviaremos el contrato a tu correo electrónico. Si es rechazado, tu contrato será anulado.</p>
      </small>
      <div class="btns-content">
        <div class="btn-tigo btn-tigo--white btn-cancel">CANCELAR</div>
        <div class="btn-tigo btn-tigo--blue btn-accept">ACEPTAR</div>
      </div>';

    // Attached.
    $form['#attached']['library'] = [
      'tigo/tigo.exists',
      'tigo/tigo.loader',
      'tigo/tigo.modal',
      'tigo/tigo.tooltip',
      'tigo_portability/tigo_portability',
    ];

    // Loader.
    $form['loader'] = [
      '#prefix' => '<div class="loader_tigo">',
      '#suffix' => '</div>',
      '#markup'  =>
      '<div class="group_loader">
          <div class="loader-ring-light"></div>
          <div class="loader-ring-track"></div>
      </div>
      <h3>Un momento por favor.</h3>',
    ];

    $form['msisdn_other'] = [
      '#type' => 'tel',
      '#title' => $this->t('Número de línea del otro operador'),
      '#placeholder' => $this->t('Número de línea del otro operador'),
      '#title_display' => 'after',
      '#maxlength' => 14,
      '#required' => TRUE,
      '#attributes' => [
        'data-format-movil' => TRUE,
        'data-number' => '0',
        'pattern' => '[\(]\d{3}[\)] \d{3}-\d{4}',
        'tabindex' => '1',
      ],
      '#prefix' => '<div class="tigo-portability-form"><div class="cardtigo cardtigo_shadow no-line"><p>Para cambiar tu número de siempre a Tigo, completa la siguiente información.</p><div class="rows"><div class="group_1">',
      '#suffix' => '<span class="error_typing error_typing_msisdn_other"></span></div>',
      '#default_value' => !empty($donorline)
      ? $donorline
      : FALSE,
      '#disabled' => !empty($donorline)
      ? $donorline
      : FALSE,
    ];

    $form['email'] = [
      '#type' => 'email',
      '#title' => $this->t('Correo electrónico'),
      '#placeholder' => $this->t('Correo electrónico'),
      '#title_display' => 'after',
      '#required' => TRUE,
      '#attributes' => [
        'pattern' => '[a-zA-Z0-9][\w\-\.]{0,}[\w]{1}@[a-zA-Z0-9][\w\-]{1,}\.[a-z]{2,4}(\.[a-z]{2})?',
        'tabindex' => '2',
      ],
      '#prefix' => '<div class="group_2">',
      '#suffix' => '<span class="error_typing error_typing_email"></span></div></div>',
      '#default_value' => !empty($email)
      ? $email
      : FALSE,
      '#disabled' => !empty($email)
      ? TRUE
      : FALSE,
    ];

    if (!empty($port_type)) {
      if ($port_type == 'PORTPOS') {
        $class = 'modalsign-contract modalsign--open';
        if ($exist_el) {
          $class .= ' hide-to-close';
        }
      }
      else {
        $class = 'modalsign-contract';
      }
    }
    else {
      $class = 'modalsign-contract';
    }

    $form['modal_content'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [$class],
      ],
    ];

    $form['modal_content']['modalsign_content'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['modalsign__content'],
      ],
    ];

    $form['modal_content']['modalsign_content']['instructions'] = [
      '#markup' => $markupModal,
    ];

    $slide = new SlideToUnlock();
    if (!empty($port_type) && $port_type == 'PORTPOS' && $exist_el) {
      $form['modal_content']['modalsign_content']['act_submit'] = [
        '#type' => 'submit',
        '#value' => 'ACEPTAR',
        '#attributes' => [
          'class' => ['buttonform request_code'],
        ],
        '#prefix' => '<div class="btn-fixed">',
        '#suffix' => '</div></div></div>',
      ];

    }
    else {
      if (isset($_SESSION['try_portability']) && $_SESSION['try_portability'] >= 3) {
        $form['content_captcha'] = [
          '#type' => 'fieldset',
          '#attributes' => [
            'id' => 'tigoune-invoice-captcha',
          ],
          '#prefix' => '<div class="btn-fixed">',
          '#suffix' => '</div></div></div>',
        ];
        $slide->add_slide($form, $form['content_captcha'], 'portabilityVtex');
      }
      else {
        $form['act_submit'] = [
          '#type' => 'submit',
          '#value' => 'CONTINUAR',
          '#attributes' => [
            'class' => ['buttonform request_code'],
          ],
          '#prefix' => '<div class="btn-fixed">',
          '#suffix' => '</div></div></div>',
        ];
      }
    }

    return $form;
  }

  /**
   * Submit formulario portabilidad.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // $result = $this->validationVtex->submitPortability($form_state->getValue('email'));.
    $msisdn = intval(preg_replace('/[^0-9]+/', '', $form_state->getValue('msisdn_other')), 10);
    $result = $this->submitPortability($form_state->getValue('email'), $msisdn);
    if (isset($result['tipo_plan']) && $result['tipo_plan'] == 'POS') {
      if ($result['new_ticket']) {
        $newTicket = 'Solicitud Ticket Nuevo';
      }
      else {
        $newTicket = 'Ticket Existente';
      }
      if ($result['tipo_plan'] != NULL) {
        $tipo_plan = $result['tipo_plan'];
      }
      if ($result['tipo_producto'] != NULL) {
        $tipo_producto = $result['tipo_producto'];
      }
      $portability_type = 'Pospago';
      $data = [
        'order_id' => $result['order_id'],
        'item_id' => $result['item_id'],
        'msisdn' => $form_state->getValue('msisdn_other'),
        'new_ticket' => $newTicket,
        'tipo plan' => $tipo_plan,
        'tipo producto' => $tipo_producto,
        'suscription_id' => $result['suscription_id'],
        'Descripción' => $result['description'],
      ];
      if ($result['ticket'] != '') {
        $data['ticket'] = $result['ticket'];
      }
      $date = date('Y-m-d H:i:s');
      $log = [
        0 => [
          'step' => 'Portabilidad ' . $portability_type,
          'log' => 'Consulta de línea',
          'data' => json_encode($data),
        ],
      ];
      $log_enconde = json_encode($log);
      Activation::queryInsertLog(
        $date,
        $result['order_id'],
        $form_state->getValue('email'),
        $log_enconde, 'Portabilidad ' . $portability_type
      );
      if ($result['new_ticket'] == FALSE) {
        $form_state->setRedirect('tigo_portability.status_vtex', [
          'order_id' => $result['order_id'],
          'item_id' => $result['item_id'],
          'new_ticket' => $result['new_ticket'],
        ]);
      }
      else {
        $form_state->setRedirect('tigo_activation.precontrato');
      }
    }
    elseif (isset($result['tipo_plan']) && $result['tipo_plan'] == 'PRE') {
      if ($result['new_ticket']) {
        $newTicket = 'Ticket Nuevo';
      }
      else {
        $newTicket = 'Ticket Existente';
      }
      if ($result['tipo_plan'] != NULL) {
        $tipo_plan = $result['tipo_plan'];
      }
      if ($result['tipo_producto'] != NULL) {
        $tipo_producto = $result['tipo_producto'];
      }
      $portability_type = 'Prepago';
      $data = [
        'order_id' => $result['order_id'],
        'item_id' => $result['item_id'],
        'msisdn' => $form_state->getValue('msisdn_other'),
        'new_ticket' => $newTicket,
        'tipo plan' => $tipo_plan,
        'tipo producto' => $tipo_producto,
        'suscription_id' => $result['suscription_id'],
        'Descripción' => $result['description'],
      ];
      if ($result['ticket'] != '') {
        $data['ticket'] = $result['ticket'];
      }
      $date = date('Y-m-d H:i:s');
      $log = [
        0 => [
          'step' => 'Portabilidad ' . $portability_type,
          'log' => 'Consulta de línea',
          'data' => json_encode($data),
        ],
      ];

      $log_enconde = json_encode($log);
      Activation::queryInsertLog(
        $date,
        $result['order_id'],
        $form_state->getValue('email'),
        $log_enconde, 'Portabilidad ' . $portability_type
      );

      $form_state->setRedirect('tigo_portability.status_vtex', [
        'order_id' => $result['order_id'],
        'item_id' => $result['item_id'],
        'new_ticket' => $result['new_ticket'],
      ]);
    }

    $helpers = new HelpersController();
    $segment = $helpers->segment_instance();
    $response = $segment->tracking_data_segment_generic(
        $data['msisdn'],
        "ET - Portabilidad",
        [
          'category' => 'Portabilidad - Correo',
          'event' => "ET - Ingresar correo",
          'label' => 'Continuar',
        ]
        );
    // }.
  }

  /**
   * Validacion formulario portabilidad.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // $this->try();
    // Se cambia la validación con el nuevo modelo.
    $msisdn = intval(
      preg_replace('/[^0-9]+/',
      '',
      $form_state->getValue('msisdn_other')),
      10
    );
    $portability_order = $this->portability->getOrderByDonorId(
      $msisdn,
      $form_state->getValue('email')
    );
    if (isset($portability_order->error) && !empty($portability_order->error)) {
      $form_state->setErrorByName(
        $portability_order->error['field'],
        $portability_order->error['message']
      );
      if (isset($_SESSION['try_portability']) &&
        $_SESSION['try_portability'] >= 3
      ) {
        drupal_set_message($portability_order->error['message'], 'error');
        global $base_url;
        if (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) &&
          $_SERVER['SERVER_NAME']
        ) {
          $base_url = $_SERVER['HTTP_X_FORWARDED_PROTO'] . '://' . $_SERVER['SERVER_NAME'];
        }
        header('Location: ' . $base_url . '/servicios/portabilidad');
        exit;
      }
    }
  }

  /**
   * Incremento intentos de consulta portabilidad.
   */
  public function try() {
    $try = isset($_SESSION['try_portability']) &&
      is_numeric($_SESSION['try_portability'])
      ? $_SESSION['try_portability']
      : 0;
    $try++;
    $_SESSION['try_portability'] = $try;
  }

  /**
   * Submit poratilidad.
   *
   * Agregado para la migración a la nueva estructura de BD.
   */
  public function submitPortability($email, $lineaDonante) {
    $result = '';
    $dataPortability = $this->portability->getOrderDetailByEmail(
      $email,
      $lineaDonante
    );
    if (isset($dataPortability[0]->tipo_plan) &&
      $dataPortability[0]->tipo_plan == 'POS' &&
      isset($dataPortability[0]->tipo_producto) &&
      $dataPortability[0]->tipo_producto == 'PORT'
    ) {
      $portability_type = 'Pospago';
      if ($dataPortability[0]->ticket != '') {
        $result = [
          'order_id' => $dataPortability[0]->order_id,
          'item_id' => $dataPortability[0]->item_id,
          'new_ticket' => FALSE,
          'tipo_producto' => $dataPortability[0]->tipo_producto,
          'tipo_plan' => $dataPortability[0]->tipo_plan,
          'description' => $dataPortability[0]->description,
          'suscription_id' => $dataPortability[0]->suscription_id,
          'ticket' => $dataPortability[0]->ticket,
        ];
      }
      elseif (isset($dataPortability[0]->imsi)
        && isset($dataPortability[0]->msisdn)
        && $dataPortability[0]->ticket == '') {
        $portacionPospago = new PortPospago();
        $result = $portacionPospago->initPortPospago($dataPortability);
      }
      else {
        $this->redirectError();
      }
    }
    elseif (isset($dataPortability[0]->tipo_plan) &&
      $dataPortability[0]->tipo_plan == 'PRE' &&
      isset($dataPortability[0]->tipo_producto) &&
      $dataPortability[0]->tipo_producto == 'PORT'
    ) {
      $portability_type = 'Prepago';
      if ($dataPortability[0]->ticket != '') {
        $result = [
          'order_id' => $dataPortability[0]->order_id,
          'item_id' => $dataPortability[0]->item_id,
          'new_ticket' => FALSE,
          'tipo_producto' => $dataPortability[0]->tipo_producto,
          'tipo_plan' => $dataPortability[0]->tipo_plan,
          'description' => $dataPortability[0]->description,
          'suscription_id' => $dataPortability[0]->suscription_id,
          'ticket' => $dataPortability[0]->ticket,
        ];
      }
      elseif (isset($dataPortability[0]->imsi)
        && isset($dataPortability[0]->msisdn)
        && $dataPortability[0]->ticket == '') {
        $portacionPrepago = new PortPrepago();
        $result = $portacionPrepago->initPortPrepago($dataPortability);
      }
      else {
        $this->redirectError();
      }
    }
    return $result;
  }

  /**
   * Se usa para hacer una redirección cuando no hay imsi ni msisdn en el item.
   */
  public function redirectError() {
    drupal_set_message(
      $this->t('El número aún no está listo para ser activado.'),
      'error'
    );
    global $base_url;
    if (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['SERVER_NAME']) {
      $base_url = $_SERVER['HTTP_X_FORWARDED_PROTO'] . '://' . $_SERVER['SERVER_NAME'];
    }
    header('Location: ' . $base_url . '/servicios/portabilidad');
    exit;
  }

}
