<?php

namespace Drupal\tigo_portability\Form;

use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\tigo\Model\SlideToUnlock;
use Drupal\tigo_portability\ValidationPortin;

class PortabilityStatus extends FormBase
{

    private $hash;

    public function __construct()
    {
        $this->hash = session_id() . 'portin_status';
    }

    public function getFormId() {
        return 'tigo_portability_status';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {
        $id_input = 'number';
        $form[$id_input] = [
            '#type' => 'tel',
            '#title' => $this->t('Número de línea de tu operador actual'),
            '#placeholder' => $this->t('Número de línea de tu operador actual'),
            '#title_display' => 'after',
            '#maxlength' => 14,
            '#required' => TRUE,
            '#attributes' => [
                'data-format-movil' => TRUE,
                'data-number' => '0',
                'pattern' => '[\(]\d{3}[\)] \d{3}-\d{4}'
            ],
            '#suffix' => '<span class="error_typing error_typing_' . $id_input . '"></span>'
        ];

        $slide = new SlideToUnlock();
        if($this->showCaptcha()){
            $form['content_captcha'] = [
                '#type' => 'fieldset',
                '#attributes' => [
                    'id' => 'tigoune-invoice-captcha',
                ]
            ];
            $form['content_captcha']['message'] = [
                '#markup' => '<p class="message_recapcha">Para continuar con la consulta, ayúdanos a validar que no eres un robot.</p>'
            ];
            $slide->add_slide($form, $form['content_captcha']['message'], 'portability_status');
        }else{
            $form['consult'] = [
                '#type' => 'submit',
                '#value' => t('Consultar'),
                '#submit' => [
                    '::submitForm'
                ],
                '#attributes' => [
                    'class' => [
                        'buttonform'
                    ]
                ],
                '#ajax' => [
                    'callback' => '::get_invoices_user_callback',
                    'event' => 'click',
                ]
            ];
        }




        $form['#attributes']['novalidate'] = '';
        $form['#attributes']['autocomplete'] = 'off';

        return $form;
    }



    public function submitForm(array &$form, FormStateInterface $form_state)
    {
    }

    public function get_invoices_user_callback(array $form, FormStateInterface &$form_state)
    {
        // incrementa en 1 el intento
        $this->setRetryQuery();
        $response = new AjaxResponse();

        $data = [
            'number' => $form_state->getValue('number')
                ? $form_state->getValue('number')
                : FALSE
        ];
        $validation = new ValidationPortin($data);
        $validate = $validation->validate_portin_status();
        $validation->segment();
        if($validate['error']) {
            $response->addCommand(new InvokeCommand(NULL, 'showAlert', [
                $validate['error'],
                'error'
            ]));
        }else{
            $alias = \Drupal::service('path.alias_manager')->getAliasByPath('/servicios/portabilidad/estatus');
            $redirect = new RedirectCommand($alias);
            $response->addCommand($redirect);
        }

        return $response;
    }

    public function setRetryQuery()
    {
        $data = \Drupal::cache()->get($this->hash);
        if ($data) {
            $retry = intval($data->data['retry']) + 1;
            \Drupal::cache()->set($this->hash, [
                'retry' => $retry
            ], REQUEST_TIME + 36000);
        } else {
            \Drupal::cache()->set($this->hash, [
                'retry' => 1
            ], REQUEST_TIME + 36000);
        }
    }

    public function showCaptcha()
    {
        $result = false;
        $data = \Drupal::cache()->get($this->hash);
        if ($data && $data->data['retry'] > 2) {
            $result = TRUE;
        }
        return $result;
    }
}
