<?php

namespace Drupal\tigo_portability\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Url;
use Drupal\Core\Render\Markup;
use Drupal\tigo\Model\SlideToUnlock;
use Drupal\tigo_portability\ValidationPortin;
use Drupal\tigo_helpers\Controller\HelpersController;

/**
 *
 */
class PortabilityStep1 extends FormBase {
  private $hash;

  /**
   * Constructor de la clase.
   */
  public function __construct() {
    $this->hash = isset($_COOKIE['portin_id'])
            ? $_COOKIE['portin_id']
            : $this->createSessionId();

  }

  /**
   * Genera una sesión de usuario.
   */
  public function createSessionId() {
    $id = session_id();
    setcookie('portin_id', $id, time() + 3600);
    return $id;
  }

  /**
   * Definición del id del formulario.
   */
  public function getFormId() {
    return 'tigo_portability_step1';
  }

  /**
   * Construcción del formulario.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Attached.
    $form['#attached']['library'] = [
      'tigo/tigo.exists',
      'tigo/tigo.loader',
      'tigo/tigo.modal',
      'tigo/tigo.tooltip',
      'tigo_portability/tigo_portability',
    ];

    // Loader.
    $form['loader'] = [
      '#prefix' => '<div class="loader_tigo">',
      '#suffix' => '</div>',
      '#markup'  =>
      '<div class="group_loader">
                    <div class="loader-ring-light"></div>
                    <div class="loader-ring-track"></div>
                </div>
                <h3>Un momento por favor.</h3>',
    ];

    $default_value = !empty(\Drupal::request()->query->get('data'))
            ? \Drupal::request()->query->get('data')
            : FALSE;
    if ($default_value) {
      $default_value = unserialize(base64_decode($default_value));
    }

    $send = $form_state->get('send');
    $form['return'] = [
      '#type' => 'link',
      '#title' => t('Regresar'),
      '#url' => Url::fromRoute(
            'tigo_portability.main'
      ),
      '#attributes' => [
        'class' => ["link-return"],
      ],

    ];
    $form['title'] = [
      '#markup' => "<h3>Pásate a tigo</h3>",
    ];
    $form['container_1'] = [
      '#type' => 'container',
      '#attributes' => array(
        'class' => 'cardtigo cardtigo_shadow no-line',
      ),
    ];
    $form['container_1']['title_2'] = [
      '#markup' => "<h4>Ingresa los siguientes datos para iniciar con tu solicitud</h4>",
    ];
    $form['container_1']['ready_code'] = [
      '#type' => 'checkbox',
      '#title' => 'Ya tengo un código de verificación (NIP) para iniciar mi solicitud',
      '#ajax' => [
        'callback' => '::wrapperCode',
        'wrapper' => 'portability-pin',
      ],
      '#default_value' => $default_value['checked']
      ? $default_value['checked']
      : FALSE,
    ];
    $form['container_1']['container_2'] = [
      '#type' => 'container',
      '#attributes' => array(
        'class' => 'rows',
      ),
    ];

    $form['container_1']['container_2']['container_3'] = [
      '#type' => 'container',
      '#attributes' => array(
        'class' => 'row_1',
      ),
    ];

    $id_input_1 = 'msisdn_tigo';
    $form['container_1']['container_2']['container_3'][$id_input_1] = array(
      '#type' => 'tel',
      '#maxlength' => 14,
      '#title' => $this->t('Número de línea Tigo'),
      '#placeholder' => $this->t('Número de línea Tigo'),
      '#title_display' => 'after',
      '#required' => TRUE,
      '#attributes' => [
        'data-format-movil' => TRUE,
        'data-number' => '0',
        'pattern' => '[\(]\d{3}[\)] \d{3}-\d{4}',
        'tabindex' => '1',
      ],
      '#default_value' => isset($default_value['msisdn'])
      ? $this->formatNumberPhone($default_value['msisdn'])
      : FALSE,
      '#prefix' => '<div class="group_1">',
      '#suffix' => '<span class="error_typing error_typing_' . $id_input_1 . '"></span></div>',
    );
    $form['container_1']['container_2']['container_3']['container_5'] = [
      '#type' => 'container',
      '#attributes' => array(
        'class' => 'sub_row',
      ),
    ];

    $form['container_1']['container_2']['container_3']['container_5']['group_custom_select'] = [
      '#type' => 'container',
      '#attributes' => array(
        'class' => 'group_custom_select',
      ),
    ];

    $form['container_1']['container_2']['container_3']['container_5']['group_custom_select']['custom_select'] = [
      '#markup' => '
                <div class="group_input_custom_select">
                    <div class="input_custom_select" data-value="CC">CC
                    </div>
                    <div class="label">Tipo</div>
                </div>
                <ul class="custom_select">
                    <li data-value="CC" class="active"><span>CC</span></li>
                    <li data-value="CE" ><span>CE</span></li>
                </ul>',
    ];

    $form['container_1']['container_2']['container_3']['container_5']['group_custom_select']['document_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Tipo'),
      '#options' => [
        'CC' => 'CC',
        'CE' => 'CE',
      ],
    ];

    $id_input_3 = 'document_number';
    $form['container_1']['container_2']['container_3']['container_5'][$id_input_3] = array(
      '#type' => 'textfield',
      '#maxlength' => 13,
      '#title' => $this->t('Número de documento'),
      '#placeholder' => $this->t('Número de documento'),
      '#title_display' => 'after',
      '#required' => TRUE,
      '#attributes' => [
        'data-format-document' => TRUE,
        'data-number' => '0',
        'pattern' => '[\d\.]{1,13}',
        'tabindex' => '2',
        'class' => ['form-document'],
      ],
      '#default_value' => isset($default_value['document'])
      ? number_format($default_value['document'], 0, '', '.')
      : FALSE,
      '#prefix' => '<div class="group_2">',
      '#suffix' => '<span class="error_typing error_typing_' . $id_input_3 . '"></span></div>',
    );
    $form['container_1']['container_2']['container_4']['container'] = [
      '#type' => 'container',
      '#attributes' => array(
        'class' => 'filter-container-status',
      ),
      '#prefix' => '<div id="portability-pin">',
      '#suffix' => '</div>',
    ];

    $form['container_1']['container_2']['container_4'] = [
      '#type' => 'container',
      '#attributes' => array(
        'class' => 'row_2',
      ),
    ];

    $id_input_2 = 'msisdn_other';
    $form['container_1']['container_2']['container_4'][$id_input_2] = array(
      '#type' => 'tel',
      '#maxlength' => 14,
      '#title' => $this->t('Número del otro operador'),
      '#placeholder' => $this->t('Número del otro operador'),
      '#title_display' => 'after',
      '#required' => TRUE,
      '#attributes' => [
        'data-format-movil' => TRUE,
        'data-number' => '0',
        'pattern' => '[\,(]\d{3}[\)] \d{3}-\d{4}',
        'tabindex' => '3',
      ],
      '#prefix' => '<div class="group_1">',
      '#suffix' => '<span class="error_typing error_typing_' . $id_input_2 . '"></span></div>',
    );

    $form['container_1']['container_2']['container_4']['container'] = [
      '#type' => 'container',
      '#attributes' => array(
        'class' => 'filter-container-status',
      ),
      '#prefix' => '<div id="portability-pin">',
      '#suffix' => '</div>',
    ];
    $ready_code = $form_state->getValue('ready_code');
    $end_process = FALSE;
    if ($send) {
      $end_process = TRUE;
    }
    elseif (isset($ready_code)) {
      if ($form_state->getValue('ready_code', NULL) === 1) {
        $end_process = TRUE;
      }
    }
    elseif (isset($default_value['checked']) && $default_value['checked']) {
      $end_process = TRUE;
    }

    if ($end_process) {

      $form['container_1']['container_2']['container_4']['container']['nip'] = array(
        '#prefix' => '<div class="container_nip">',
        '#suffix' => '<span class="icon_info tooltip_js_data" data-messages="Ingresa los 5 dígitos del código de verificación (NIP) enviado como SMS a tu celular." data-direction="topRight"></span></div>',
        '#type' => 'textfield',
        '#maxlength' => 5,
        '#title' => $this->t('Código de verificación (NIP)'),
        '#placeholder' => $this->t('Código de verificación (NIP)'),
        '#title_display' => 'after',
        '#max' => 99999,
        '#attributes' => [
          'data-int-number' => TRUE,
          'pattern' => '[\d]{1,5}',
          'tabindex' => '4',
        ],
      );
      $try = $form_state->get('try');
      if ($try > 0) {
        $messageTry = $this->convertNumber($try);
        $form['container_1']['container_2']['container_4']['container']['recode'] = [
          '#prefix' => $messageTry,
          '#type' => 'submit',
          '#value' => "Reenviar código",
          '#submit' => ['::submitGetCode'],
          '#ajax' => [
            'callback' => '::wrapperCode',
            'wrapper' => 'portability-pin',
          ],
          '#attributes' => [
            'class' => [
              'button-recode',
            ],
          ],
        ];
      }
      if ($this->showCaptcha()) {
        $slide = new SlideToUnlock();
        $slide->add_slide($form, $form['container_1']['container_2']['container_4']['container'], 'portability');
      }
      else {
        $form['container_1']['container_2']['container_4']['container']['submit'] = [
          '#type' => 'submit',
          '#value' => "Finalizar Solitud",
          '#attributes' => [
            'class' => [
              'buttonform',
            ],
          ],
          '#validate' => [
            '::validateForm',
            '::validateCode',
          ],
        ];
      }
    }
    else {
      $form['container_1']['container_2']['container_4']['container']['code'] = [
        '#type' => 'submit',
        '#value' => "Solicitar código",
        '#submit' => ['::submitGetCode'],
        '#attributes' => [
          'class' => [
            'buttonform',
            'request_code',
          ],
        ],
        '#ajax' => [
          'callback' => '::wrapperCode',
          'wrapper' => 'portability-pin',
        ],
      ];
    }
    $form_state->setCached(FALSE);

    $form['#attributes']['novalidate'] = '';
    $form['#attributes']['autocomplete'] = 'off';

    return $form;
  }

  /**
   *
   */
  public function wrapperCode(array &$form, FormStateInterface $form_state) {
    return $form['container_1']['container_2']['container_4']['container'];
  }

  /**
   *
   */
  public function submitGetCode(array &$form, FormStateInterface $form_state) {
    $msisdn_other = $form_state->getValue('msisdn_other')
            ? intval(preg_replace('/[^0-9]+/', '', $form_state->getValue('msisdn_other')), 10)
            : FALSE;
    $service = \Drupal::service('tigo_portability.operator_donor');
    $data['msisdn'] = $msisdn_other;
    $service->operator($data);
    $response = $service->getResponse();
    if ($response->payload->donorId) {
      $service_pin = \Drupal::service('tigo_portability.pin');

      $data['msisdn'] = $msisdn_other;
      $data['donorid'] = $response->payload->donorId;
      $service_pin->pin($data);
      $response = $service_pin->getResponse();
      if (is_null($response)) {
        $re = '/([\d]{3})([\d]{3})([\d]{4})/';
        $str = $msisdn_other;
        $subst = '($1) $2-$3';
        $result = preg_replace($re, $subst, $str);
        $helpers = new HelpersController();
        $segment = $helpers->segment_instance();
        $response = $segment->tracking_data_segment_generic(
        $result,
        "PT - Portabilidad",
        [
          'category' => 'Portabilidad - NIP ',
          'event' => "PT - Solicitar NIP",
          'label' => 'Iniciar Solicitud',
          'msisdn' => $result,
        ]
        );
        drupal_set_message('Se ha enviado a la línea ' . $result . ' un mensaje de texto con el código de verificación (NIP). Por favor ingrésalo.', 'info');
      }
    }
    $try = $form_state->get('try');
    if ($try) {
      $try -= 1;
      $form_state->set('try', $try);
    }
    else {
      $form_state->set('try', 3);
    }
    $form_state->set('send', 1);
    $form_state->setRebuild();
  }

  /**
   *
   */
  public function validateCode(array &$form, FormStateInterface $form_state) {
    $data = [
      'nip' => $form_state->getValue('nip')
      ? $form_state->getValue('nip')
      : FALSE,
    ];
    $validation = new ValidationPortin($data);
    $validation->validateCode();
    $errors = $validation->getValidation();
    $validation->segment();
    if ($errors) {
      foreach ($errors as $key => $error) {
        $form_state->setErrorByName($key, $error);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $data = [
      'msisdn_tigo' => $form_state->getValue('msisdn_tigo')
      ? $form_state->getValue('msisdn_tigo')
      : FALSE,
      'document_number' => $form_state->getValue('document_number')
      ? $form_state->getValue('document_number')
      : FALSE,
      'document_type' => $form_state->getValue('document_type')
      ? $form_state->getValue('document_type')
      : FALSE,
      'msisdn_other' => $form_state->getValue('msisdn_other')
      ? $form_state->getValue('msisdn_other')
      : FALSE,

    ];
    $validation = new ValidationPortin($data);
    $validation->validationForm();
    $errors = $validation->getValidation();
    $validation->segment();
    if ($errors) {
      foreach ($errors as $key => $error) {
        $form_state->setErrorByName($key, $error);
      }
    }
  }

  /**
   *
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->setRetryQuery();
    $data = [
      'msisdn_other' => $form_state->getValue('msisdn_other'),
      'msisdn_tigo' => $form_state->getValue('msisdn_tigo'),
      'nip' => $form_state->getValue('nip'),
      'document_number' => $form_state->getValue('document_number'),
      'document_type' => $form_state->getValue('document_type'),
    ];

    $portability = new ValidationPortin($data);
    $portin = $portability->portin();
    $portability->segment();
    if (isset($portin['send'])) {
      $form_state->setRedirect('tigo_portability.status');
    }
    else {
      drupal_set_message(Markup::create(
        $portin['error']
      ), 'error');
    }
  }

  /**
   *
   */
  public function convertNumber($try) {
    $message = FALSE;
    switch ($try) {
      case 1:
        $message = '<span>¿Aún no recibes el código? </span><b>Te queda un intento</b>';
        break;

      case 2:
        $message = '<span>¿Aún no recibes el código? </span><b>Te quedan dos intentos</b>';
        break;

      default:
        $message = '<span>¿No has recibido el código? </span>';
        break;
    }

    return $message;
  }

  /**
   *
   */
  public function setRetryQuery() {
    $data = \Drupal::cache()->get($_COOKIE['portin_id']);
    if ($data) {
      $retry = intval($data->data['retry']) + 1;
      \Drupal::cache()->set($_COOKIE['portin_id'], [
        'retry' => $retry,
      ], REQUEST_TIME + 36000);
    }
    else {
      \Drupal::cache()->set($_COOKIE['portin_id'], [
        'retry' => 1,
      ], REQUEST_TIME + 36000);
    }
  }

  /**
   *
   */
  public function showCaptcha() {
    $result = FALSE;
    $data = \Drupal::cache()->get($_COOKIE['portin_id']);
    if ($data && $data->data['retry'] > 2) {
      $result = TRUE;
    }
    return $result;
  }

  /**
   *
   */
  public function formatNumberPhone($msisdn) {
    $re = '/([\d]{3})([\d]{3})([\d]{4})/';
    $str = $msisdn;
    $subst = '($1) $2-$3';

    $result = preg_replace($re, $subst, $str);
    return $result;
  }

}
