<?php

namespace Drupal\tigo_portability\Form;

use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\tigo_helpers\Controller\HelpersController;
use Drupal\tigo_portability\Model\ZendeskTicket;
use Drupal\tigo_portability\Services\GestorSuscripcion;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\tigo_portability\Model\PortabilityModel;
use Drupal\tigo_activation\Services\ChangeStatus;
use Drupal\tigo_activation\Activation;

/**
 * Formulario activacion asesor portacion.
 */
class SupportStatus extends FormBase {
  /**
   * Variable para formatear conexion con la base de datos.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;
  /**
   * Variable para guardar variable msisdn.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $msisdn;
  /**
   * Variable para formatear fecha.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormat;
  /**
   * Variable actualizar campos en el tikect.
   *
   * @var \Drupal\tigo_portability\Model\ZendeskTicket
   */
  protected $zendeskTicket;
  /**
   * Variable para consultar datos del admin.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;
  /**
   * Variable para consultar datos del admin.
   *
   * @var \Drupal\tigo_portability\Services\GestorSuscripcion
   */
  protected $gestor;
  /**
   * The link relation type manager used to create HTTP header links.
   *
   * @var \Drupal\tigo_activation\Services\ChangeStatus
   */
  protected $changeStatus;
  /**
   * The model for Portability entity integration.
   *
   * @var \Drupal\tigo_portability\Model\PortabilityModel
   */
  protected $portability;
  /**
   * The global drupal object.
   *
   * @var \Drupal
   */
  protected $drupal;

  /**
   * Contructor del formulario.
   */
  public function __construct(
    Connection $connection,
    DateFormatter $dateFormat,
    ZendeskTicket $zendesk_ticket,
    AccountProxyInterface $user,
    GestorSuscripcion $gestor_suscripcion,
    PortabilityModel $portability_model,
    ChangeStatus $change_status,
    $drupal = NULL) {
    $this->database = $connection;
    $this->dateFormat = $dateFormat;
    $this->zendeskTicket = $zendesk_ticket;
    $this->currentUser = $user;
    $this->gestor = $gestor_suscripcion;
    $this->portability = $portability_model;
    $this->changeStatus = $change_status;
    $this->drupal = $drupal;
    if (!is_object($this->drupal)) {
      $this->drupal = new \Drupal();
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('date.formatter'),
      $container->get('tigo_portability.zendesk_ticket'),
      $container->get('current_user'),
      $container->get('tigo_portability.package_gestor'),
      $container->get('tigo_portability.portability_model'),
      $container->get('tigo_activation.change_status')
    );
  }

  /**
   * Returns a unique string identifying the form.
   *
   * The returned ID should be a unique string that can be a valid PHP function
   * name, since it's used in hook implementation names such as
   * hook_form_FORM_ID_alter().
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'tigo_portability_support_status';
  }

  /**
   * Form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $msisdn = NULL) {
    $this->msisdn = $msisdn;
    $data = $this->msisdn;
    $query = $this->database->select('tigo_suscripcion_items', 'tsi');
    $query->fields('tsi', ['item_id', 'suscription_id']);
    $query->fields('tsz', ['ticket']);
    $query->condition('tsi.order_id', $data['order_id'], '=');
    $query->condition('tsi.item_id', $data['item_id'], '=');
    $query->join('tigo_suscripcion_zendesk', 'tsz', 'tsi.item_id = tsz.item_id');
    $result = $query->execute()->fetchObject();

    $dataInfo = !empty($result->data_submit)
      ? json_decode($result->data_submit)
      : FALSE;

    $form['zendesk_ticket'] = [
      '#type' => 'hidden',
      '#value' => $result->ticket,
    ];

    $form['observaciones'] = [
      '#type' => 'textarea',
      '#required' => TRUE,
      '#title' => 'Observaciones',
    ];

    if (isset($data['typePlan']) && $data['typePlan'] == 'POS') {
      $form['fecha'] = [
        '#type' => 'date',
        '#title' => 'Fecha Activación',
        '#default_value' => !empty($dataInfo->fecha)
        ? $dataInfo->fecha
        : FALSE,
        '#required' => TRUE,
      ];

      $form['send_mail'] = [
        '#type' => 'checkbox',
        '#title' => 'Enviar correo electrónico con el contrato',
        '#default_value' => 1,
      ];
      $infoBtn = ' y generar pdf';
    }
    else {
      $infoBtn = '';
    }
    if ($data['msisdnAsig']) {
      $form['buttons']['assign_resource'] = [
        '#type' => 'submit',
        '#value' => 'Asignar recursos' . $infoBtn,
      ];
    }
    $form['buttons']['ported_cancel'] = [
      '#type' => 'submit',
      '#value' => 'Cancelar',
      '#submit' => ['::submitCancelPort'],
    ];

    return $form;
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $helpers = new HelpersController();
    $data = $this->queryOrders($form, $form_state);
    $result = [];
    $content = $this->msisdn;
    $base_url = "https://transacciones.tigo.com.co";
    $params = [];

    if (isset($content['typePlan']) && $content['typePlan'] == 'POS') {
      /*
       * Genero el pdf.
       */
      if ($data) {
        $this->changeStatus->callbackActivation(
          'WSEC' . $data[0]->item_id,
          100,
          $content['msisdnAsig'],
          'Generacion contrato exitoso',
          $form_state->getValue('send_mail'),
          FALSE
        );
      }
      $portability_type = 'Pospago';
      $messageStatus = 'Se ha creado el contrato exitosamente.';
      $jsonRequest = [
        'messageSatus' => 'Se ha generado el pdf',
        'classSatus' => 'succes',
        'logMessage' => 'Se ha creado en contrato pdf y enviado al correo',
        'respuesta' => '',
        'dataInfo' => [
          'msisdn' => $content['msisdnAsig'],
          'ref_id' => $content['ref_id'],
          'ticket' => $form_state->getValue('zendesk_ticket'),
          'pathPdf' => '',
        ],
      ];
      $result = [
        'item_id' => $data[0]->item_id,
        'updated' => date('Y-m-d H:i:s'),
        'status' => 'CLOSE',
        'status_detail' => 'GENERATE pdf',
        'package_id' => $content['ref_id'],
        'msisdn' => $content['msisdnAsig'],
        'portado' => $content['portado'],
        'json_request' => json_encode($jsonRequest),
        'json_response' => '-',
        'response_code' => '100',
        'response_message' => 'Se ha creado en contrato pdf y enviado al correo',
        'asesor_id' => $this->currentUser->id(),
        'observaciones' => $form_state->getValue('observaciones'),
        'plan_type' => $data[0]->tipo_plan,
      ];
    }
    else {
      $portability_type = 'Prepago';
      if ($data[0]->status == 'OPEN') {
        $result = $this->updateStatusZendezk($content['ref_id'], $content['msisdnAsig'], $form_state->getValue('zendesk_ticket'));
      }
      $result['item_id'] = $data[0]->item_id;
      $result['asesor_id'] = $this->currentUser->id();
      $result['observaciones'] = $form_state->getValue('observaciones');
      $result['plan_type'] = $data[0]->tipo_plan;
    }

    $updateZendek = $this->zendeskTicket->updatePortabilityTicket($form_state->getValue('zendesk_ticket'), $result);

    $asesor = $this->currentUser;
    $log = [
      'step' => '3: Portabilidad ' . $portability_type,
      'log' => $messageStatus,
      'data' => json_encode([
        'asesor_id' => $asesor->id(),
        'Nombre asesor' => $asesor->getUsername(),
        'Numero de la orden' => $data[0]->order_id,
        'Ticket' => $data[0]->ticket,
        'Descripción' => $data[0]->description,
        'Nombre cliente' => $data[0]->client_name,
        'Documento cliente' => $data[0]->client_document,
        'MSISDN nuevo' => $data[0]->msisdn,
        'Donorid' => $data[0]->linea_donante,
        'item_id' => $data[0]->item_id,
        'Status' => $result['status'],
        'Observaciones' => $form_state->getValue('observaciones'),
      ]),
    ];
    $log_encode = json_encode([$log]);
    $date = date('Y-m-d H:i:s');
    Activation::queryInsertLog(
      $date,
      $data[0]->order_id,
      $data[0]->client_email,
      $log_encode,
      'Portabilidad Prepago'
    );

    if ($updateZendek) {
      $form_state->setRedirect('tigo_portability.ticket_ported_step2', [
        'ref_id' => $content['ref_id'],
        'msisdn' => $content['msisdnAsig'],
        'ticket' => $form_state->getValue('zendesk_ticket'),
      ]
      );
    }

    $segment = $helpers->segment_instance();
    $segment->tracking_data_segment_generic(
      $content['msisdnAsig'],
      "ET - Asignar recursos",
      [
        'category' => 'Portabilidad - Backend',
        'event' => "ET - Asignar recursos",
        'label' => "Confirmación",
      ]
    );
  }

  /**
   * Funcion para cancelar la portacion.
   */
  public function submitCancelPort(array &$form, FormStateInterface $form_state) {
    $helpers = new HelpersController();
    $data = $this->queryOrders($form, $form_state);
    $content = $this->msisdn;
    $jsonRequest = [
      'messageSatus' => 'Se ha Cancelado la orden.',
      'classSatus' => 'warning',
    ];

    $datos = [
      'status' => 'CANCEL',
      'asesor_id' => $this->currentUser->id(),
      'observaciones' => $form_state->getValue('observaciones'),
    ];
    $ticket = $form_state->getValue('zendesk_ticket');

    $updateZendek = $this->zendeskTicket->updatePortabilityTicket($ticket, $datos);

    $messageStatus = $jsonRequest['messageSatus'];
    $asesor = $this->currentUser;
    if ($data[0]->tipo_plan == 'POS') {
      $portability_type = 'Pospago';
    }
    elseif ($data[0]->tipo_plan == 'PRE') {
      $portability_type = 'Prepago';
    }

    $log = [
      'step' => '3: Portabilidad ' . $portability_type,
      'log' => $messageStatus,
      'data' => json_encode([
        'asesor_id' => $asesor->id(),
        'Nombre asesor' => $asesor->getUsername(),
        'Numero de la orden' => $data[0]->order_id,
        'Ticket' => $data[0]->ticket,
        'Descripción' => $data[0]->description,
        'Nombre cliente' => $data[0]->client_name,
        'Documento cliente' => $data[0]->client_document,
        'MSISDN nuevo' => $data[0]->msisdn,
        'Donorid' => $data[0]->linea_donante,
        'item_id' => $data[0]->item_id,
        'Status' => $datos['status'],
        'Observaciones' => $form_state->getValue('observaciones'),
      ]),
    ];
    $log_encode = json_encode([$log]);
    $date = date('Y-m-d H:i:s');
    Activation::queryInsertLog(
      $date,
      $data[0]->order_id,
      $data[0]->client_email,
      $log_encode,
      'Portabilidad Prepago'
    );

    if ($updateZendek) {
      $form_state->setRedirect('tigo_portability.ticket_ported_step2', [
        'ref_id' => $content['ref_id'],
        'msisdn' => $content['msisdnAsig'],
        'ticket' => $form_state->getValue('zendesk_ticket'),
      ]);
    }

    $segment = $helpers->segment_instance();
    $segment->tracking_data_segment_generic(
      $content['msisdnAsig'],
      "ET - Asignar recursos",
      [
        'category' => 'Portabilidad - Backend',
        'event' => "ET - Cancelar solicitud",
        'label' => "Cancelar",
      ]
    );
  }

  /**
   * Obtengo la informacion.
   */
  public function queryOrders(array &$form, FormStateInterface $form_state) {
    $return = $this->portability->getItemByZendeskTicket($form_state->getValue('zendesk_ticket'));
    return $return;
  }

  /**
   * Actualizo estado del tickect de zendezk.
   */
  public function updateStatusZendezk($ref_id, $msisdn, $ticket) {
    $datos = [];
    $content = $this->msisdn;
    $response = $this->gestor->gestorCompra($msisdn, $ref_id, $ticket);
    $gestorResponse = json_decode($response->getContent());

    if (isset($gestorResponse->payload->code) && $gestorResponse->payload->code == 210) {
      // Cuando no hay una respuesta exitosa del servicio.
      $status = 'ACREDITADO';
      $messaageStatus = 'Se ha acreditado la línea';
      $logMessage = 'Se acreditan los recursos';
      $classStatus = 'succes';
      $data_request = [
        'msisdn' => $msisdn,
        'ref_id' => $ref_id,
        'ticket' => $ticket,
        'pathPdf' => '',
      ];
    }
    elseif (isset($gestorResponse->code) && $gestorResponse->code == 704) {
      // Cuando falla la comunicación con el API.
      $status = 'PENDING';
      $messaageStatus = 'La orden esta pendiente';
      $classStatus = 'warning';
      $logMessage = 'Se presenta un problema con el servicio.';
      $respuesta = $gestorResponse->message;
      $data_request = '';
    }
    else {
      $status = 'RECHAZADO';
      $messaageStatus = 'Se rechazo la orden.';
      $logMessage = 'Se rechaza la transacción';
      $classStatus = 'warning';
      $respuesta = $gestorResponse->payload->message;
      $data_request = '';
    }

    $jsonRequest = [
      'messageSatus' => $messaageStatus,
      'classSatus' => $classStatus,
      'logMessage' => $logMessage,
      'respuesta' => $respuesta,
      'dataInfo' => $data_request,
    ];
    $datos = [
      'messageSatus' => $messaageStatus,
      'updated' => date('Y-m-d H:i:s'),
      'status' => $status,
      'package_id' => $ref_id,
      'msisdn' => $msisdn,
      'portado' => $content['portado'],
      'json_request' => json_encode($jsonRequest),
      'json_response' => $response,
      'response_code' => isset($gestorResponse->payload->code)
      ? $gestorResponse->payload->code
      : $gestorResponse->code,
      'response_message' => $gestorResponse->message,
    ];
    return $datos;
  }

}
