<?php

namespace Drupal\tigo_portability\Model;

use Drupal\Core\Database\Connection;

/**
 * Clase modelo para las conexiones a base de datos de tickets de zendesk.
 *
 * Se define como servicio para ser accedido en Dependency Injection.
 *
 * @package Drupal\tigo_portability
 */
class ActivationModel {

  /**
   * Nombre de las tablas.
   *
   * @var string
   */
  const ORDERS = 'tigo_suscripcion_orders';
  const ITEMS = 'tigo_suscripcion_items';
  const PORT = 'tigo_suscripcion_portability';
  const DISPATCH = 'tigo_suscripcion_dispatch';
  const EVIDENCES = 'tigo_suscripcion_evidences';
  const SERVICE = 'tigo_suscripcion_service_request';
  const ZENDESK = 'tigo_suscripcion_zendesk';

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructor de la clase.
   */
  public function __construct(Connection $connection) {
    $this->database = $connection;
  }

  /**
   * Función que busca ordenes de tipo portación en BD.
   */
  public function getOrderByOrderId($orderId) {
    $select = $this->database->select(self::ORDERS, 'tso');
    $select->fields('tso', ['order_id']);
    $select->addField('tsi', 'item_id');
    $select->condition('tso.order_id', $orderId);
    $select->condition('tsi.tipo_producto', 'LN');
    $select->join(self::ITEMS, 'tsi', 'tso.order_id = tsi.order_id');
    $result = $select->execute()->fetchAll();
    return $result;
  }

  /**
   * Función que busca item por item id.
   */
  public function getItemByItemId($itemId) {
    $select = $this->database->select(self::ITEMS, 'tsi');
    $select->fields('tsi');
    $select->condition('tsi.item_id', $itemId);
    $result = $select->execute()->fetchAll();
    return $result;
  }

}
