<?php

namespace Drupal\tigo_portability\Model;

use Drupal\Core\Database\Connection;

/**
 * Clase modelo para las conexiones a base de datos de Seller info.
 *
 * Se define como servicio para ser accedido en Dependency Injection.
 */
class SellerModel {

  /**
   * Definición de alias para las tablas tigo_suscripcion_seller_data.
   */
  const SELLER_DATA = 'tigo_suscripcion_seller_data';
  /**
   * Definición de alias para las tablas tigo_suscripcion_seller_center.
   */
  const SELLER_CENTER = 'tigo_suscripcion_seller_center';

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructor de la clase.
   *
   * Incluye diferentes inicializaciones para la inyección de dependencia.
   *
   * @param Drupal\Core\Database\Connection $connection
   *   The database connection with D. I.
   */
  public function __construct(Connection $connection) {
    $this->database = $connection;
  }

  /**
   * Consulta a la tabla tigo_suscripcion_seller_data.
   *
   * Devuelve los datos de una consulta filtrada por el profile_id.
   *
   * @param string $profile_id
   *   Se busca por un solo profile_id a la vez.
   *
   * @return array
   *   Si hay resultados devuelve datos en un arreglo.
   */
  public function getDataProfile($profile_id) {
    $query = $this->database->select(self::SELLER_DATA, 'tssd');
    $query->fields('tssd');
    $query->condition('tssd.profile_id', $profile_id);
    $result = $query->execute()->fetchAll();
    return $result;
  }

}
