<?php

namespace Drupal\tigo_portability\Model;

use Drupal\Core\Database\Connection;

/**
 * Clase modelo para las conexiones a base de datos de Token para NIP.
 *
 * Se define como servicio para ser accedido en Dependency Injection.
 *
 * @package Drupal\tigo_portability
 */
class TokenModel {

  /**
   * Definición de tablas.
   */
  const TOKEN_REQUEST = 'tigo_portability_token_request';
  const NIP_REQUEST = 'tigo_portability_nip_request';

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructor de la clase.
   */
  public function __construct(Connection $connection) {
    $this->database = $connection;
  }

  /**
   * Función que busca un token en BD.
   *
   * @param string $token
   *   Id de token a encontrar.
   *
   * @return array
   *   Retorna el resultado de la busqueda
   */
  public function getToken($token) {
    $select = $this->database->select(self::TOKEN_REQUEST, 'tptr');
    $select->fields('tptr');
    $select->condition('tptr.token', $token);
    $result = $select->execute()->fetchAll();
    return $result;
  }

  /**
   * Función que guarda un nuevo token en BD.
   *
   * @param array $data
   *   Id de token a encontrar.
   *
   * @return int
   *   Retorna el id del insert.
   */
  public function setToken(array $data) {
    $insert = $this->database->insert(self::TOKEN_REQUEST);
    $insert->fields($data);
    $result = $insert->execute();
    return $result;
  }

  /**
   * Función que guarda un nuevo log de petición.
   *
   * @param array $data
   *   Id de token a encontrar.
   *
   * @return int
   *   Retorna el id del insert.
   */
  public function setNipRequest(array $data) {
    $insert = $this->database->insert(self::NIP_REQUEST);
    $insert->fields($data);
    $result = $insert->execute();
    return $result;
  }

}
