<?php

namespace Drupal\tigo_portability\Services;

use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\tigoapi\TigoApi;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\tigo_portability\Model\ZendeskTicket;

/**
 * Servicio para el consumo de gestores.
 *
 * @package Drupal\tigo_portabilidad
 * @license https://transacciones.tigo.com.co/servicios/paquetes  License 3.01
 * @link https://transacciones.tigo.com.co/servicios/paquetes
 */
class GestorSuscripcion {

  protected $zendeskTicket;
  protected $tigoApi;
  protected $killSwitch;
  protected $configFactory;
  protected $response;

  /**
   * Constructor de la clase.
   */
  public function __construct(
    ZendeskTicket $zendesk_ticket,
    TigoApi $tigo_api,
    KillSwitch $kill_switch,
    ConfigFactoryInterface $config_factory
  ) {
    $this->zendeskTicket = $zendesk_ticket;
    $this->tigoApi = $tigo_api;
    $this->killSwitch = $kill_switch;
    $this->configFactory = $config_factory->get('tigo_portability.adminsettings');
  }

  /**
   * Función que consume el servicio de asignación de paquetes.
   *
   * @param string $msisdn
   *   Línea a asignar el paquete.
   * @param string $idPackageGestor
   *   Línea a asignar el paquete.
   * @param string $transactionId
   *   Línea a asignar el paquete.
   *
   * @return Symfony\Component\HttpFoundation\JsonResponse
   *   Retorna la respuesta del servicio
   */
  public function gestorCompra($msisdn, $idPackageGestor, $transactionId) {
    $authorization = $this->configFactory->get('authorization');
    $platformId = $this->configFactory->get('platform_id');
    $body = [
      'msisdn' => $msisdn,
      'packageId' => $idPackageGestor,
      "packageAmount" => '1',
    ];
    $params = [
      'query' => [
        'type' => 'manager',
      ],
      'body' => json_encode($body),
      'headers' => [
        'cache-control' => 'no-cache',
        'transactionid' => $transactionId,
        'usertransaction' => 'Paquetes',
        'authorization' => $authorization,
        'platformid' => $platformId,
      ],
    ];
    try {
      $this->response = $this->tigoApi->consumption(
          'vasProductSubscribe', $params, TRUE, TRUE, FALSE
      );
      return new JsonResponse($this->response);
    }
    catch (\Exception $e) {
      $mensaje = (object) [
        'code' => $e->getCode(),
        'message' => $e->getMessage(),
      ];
      return new JsonResponse($mensaje);
    }
  }

}
