<?php

namespace Drupal\tigo_portability\Services;

use Drupal\Core\Database\Connection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\tigo_portability\Model\TokenModel;

/**
 * Clase que define el Token.
 *
 * @package Drupal\tigo_portability
 * @license https://transacciones.tigo.com.co/portabilidad/get-token  License 3.01
 * @link https://transacciones.tigo.com.co/portabilidad/get-token Generate new token
 */
class Token {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Variable para traer configuraciones.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Variable para traer configuraciones.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $tokenModel;

  /**
   * Constructor de la clase.
   */
  public function __construct(
    Connection $connection,
    ConfigFactoryInterface $config_factory,
    TokenModel $token_model
  ) {
    $this->database = $connection;
    $this->configFactory = $config_factory->get('tigo_portability.adminsettings');
    $this->tokenModel = $token_model;
  }

  /**
   * Solicita un nuevo Token valido.
   *
   * @return array
   *   Con el nuevo token generado
   */
  public function getToken() {
    $token = $this->generateToken();
    //$tokenValidate = $this->validateToken($token);
    //if ($tokenValidate) {
      $respuesta['token'] = $token;
      $respuesta['code'] = 200;
      $respuesta['message'] = 'Token válido';
    // }
    // else {
    //   $respuesta['token'] = FALSE;
    //   $respuesta['code'] = 401;
    //   $respuesta['message'] = 'Token no válido';
    // }
    return $respuesta;
  }

  /**
   * Genera un token de forma aleatoria.
   *
   * @return string
   *   El valor del nuevo token
   */
  public function generateToken() {
    $source = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
    $token = substr(str_shuffle($source), 0, 30);
    $expired = time() + ($this->configFactory->get('time_available') * 60);
    $time_available = date('Y-m-d H:i:s', $expired);
    $data = [
      'token' => $token,
      'created' => date('Y-m-d H:i:s'),
      'expired' => $time_available,
      'domain' => isset($_SERVER['HTTP_REFERER'])
      ? $_SERVER['HTTP_REFERER']
      : '',
    ];
    $this->tokenModel->setToken($data);
    return $token;
  }

  /**
   * Valida existencia y expiración de un Token.
   *
   * @return bool
   *   Dependiendo la validez del token
   */
  public function validateToken($token) {
    $return = FALSE;
    $exist = $this->tokenExist($token);
    if ($exist !== FALSE) {
      $return = $this->validateExpiration($exist);
    }
    return $return;
  }

  /**
   * Función que busca la existencia de un token.
   *
   * @return object|bool
   *   Con el registro de la BD
   */
  public function tokenExist($token) {
    $return = FALSE;
    $select = $this->tokenModel->getToken($token);
    if (count($select) > 0) {
      $return = $select;
    }
    return $return;
  }

  /**
   * Función que verifica la validez de la fecha del token.
   *
   * @return bool
   *   Dependiendo de la validez
   */
  public function validateExpiration($exist) {
    $notExpired = TRUE;
    if (isset($exist[0]->expired)) {
      if (date('Y-m-d H:i:s') > $exist[0]->expired) {
        $notExpired = FALSE;
      }
    }
    else {
      $notExpired = FALSE;
    }
    return $notExpired;
  }

}
