<?php

namespace Drupal\tigo_portability;

use Drupal\tigo_helpers\Controller\HelpersController;

/**
 *
 */
class ValidationPortin {


  private $msisdn_tigo;
  private $document_number;
  private $document_type;
  private $msisdn_other;
  private $nip;
  private $number;
  private $error;
  private $return;

  /**
   *
   */
  public function __construct($data) {
    $this->msisdn_tigo = isset($data['msisdn_tigo'])
    ? intval(preg_replace('/[^0-9]+/', '', $data['msisdn_tigo']), 10)
    : FALSE;
    $this->msisdn_other = isset($data['msisdn_other'])
    ? intval(preg_replace('/[^0-9]+/', '', $data['msisdn_other']), 10)
    : FALSE;
    $this->document_number = isset($data['document_number'])
    ? intval(preg_replace('/[^0-9]+/', '', $data['document_number']), 10)
    : FALSE;
    $this->document_type = isset($data['document_type'])
    ? $data['document_type']
    : FALSE;
    $this->nip = isset($data['nip'])
    ? $data['nip']
    : FALSE;
    $this->number = isset($data['number'])
    ? intval(preg_replace('/[^0-9]+/', '', $data['number']), 10)
    : FALSE;
    $this->error = FALSE;
    $this->return = FALSE;
  }

  /**
   *
   */
  public function validationForm() {

    if (strlen($this->msisdn_tigo) != 10) {
      $this->error['msisdn_tigo'] = 'El número tigo debe ser de 10 digitos';
    }
    else {
      $service = \Drupal::service('tigo_portability.check_user');
      $data['msisdn'] = $this->msisdn_tigo;
      $service->check($data);
      $response = $service->getResponse();
      if (!isset($response->TigoApiResponse->response->ClientType->documenNumber)
      || $response->TigoApiResponse->response->ClientType->documenNumber != $this->document_number
           ) {
        $this->error['msisdn_tigo'] = 'El número de documento que ingresaste, no corresponde al titular de la línea Tigo';
      }
      if (!isset($response->TigoApiResponse->response->ClientType->documentType)
      || strcasecmp($response->TigoApiResponse->response->ClientType->documentType, $this->document_type) != 0
           ) {
        $this->error['document_type'] = 'El tipo de documento que ingresaste, no corresponde al titular de la línea Tigo';
      }
    }
    if (strlen($this->msisdn_other) != 10) {
      $this->error['msisdn_other'] = 'El número del otro operador debe ser de 10 digitos';
    }
    else {
      $service = \Drupal::service('tigo_portability.operator_donor');
      $data['msisdn'] = $this->msisdn_other;
      $service->operator($data);
      $response = $service->getResponse();
      if (isset($response->payload->donorId)
      && $response->payload->donorId == "102") {
        $this->error['msisdn_other'] = 'El número del otro operador no puede ser Tigo';
      }
      elseif (!isset($response->payload->donorId)) {
        $message = (method_exists($response, 'getMessage'))
        ? json_decode($response->getMessage())
        : $response;
        if (isset($message->payload->msgCode)) {
          $this->error['msisdn_other'] = $message->payload->description;
        }
      }
    }
    return $this;
  }

  /**
   *
   */
  public function validateCode() {
    if (!$this->nip || strlen($this->nip) != 5) {
      $this->error['nip'] = 'El número NIP debe ser de 5 digitos';
    }
    return $this;
  }

  /**
   *
   */
  public function getValidation() {
    return $this->error;
  }

  /**
   *
   */
  public function portin() {
    $portin = FALSE;
    $service = \Drupal::service('tigo_portability.portin');
    $data = [
      'msisdn_other' => $this->msisdn_other,
      'msisdn_tigo' => $this->msisdn_tigo,
      'nip' => $this->nip,
      'idenfitication_number' => $this->document_number,
      'idenfitication_type' => $this->document_type,
    ];
    $service->port($data);
    $response = $service->getResponse();
    ksm($response);
    $message = '<b>Lo sentimos</b>, algo salió mal mientras confirmabamos tus datos. Por favor ingrésalos de nuevo.';
    if ($response) {
      $get_message = (method_exists($response, 'getMessage'))
      ? json_decode($response->getMessage())
      : $response;
      if ($get_message->payload->msgCode) {
        $this->return['error'] = $message;
      }
      else {
        $_SESSION['msisdn_portability'] = $this->msisdn_other;
        $this->return['send'] = TRUE;
      }
    }
    else {
      $this->return['error'] = $message;
    }
    return $this->return;
  }

  /**
   *
   */
  public function validate_portin_status() {
    if (strlen($this->number) != 10 || !is_numeric($this->number)) {
      $this->return['error'] = 'El número de línea que ingresaste no es válido, Verficalo e intenta de nuevo';
    }
    else {
      $_SESSION['msisdn_portability'] = $this->number;
      $this->return['send'] = TRUE;
    }
    return $this->return;
  }

  /**
   *
   */
  public function segment() {
    $helpers = new HelpersController();
    $segment = $helpers->segment_instance();
    if ($this->error && is_array($this->error)) {
      foreach ($this->error as $error) {
        $response = $segment->tracking_data_segment_generic(
        $this->msisdn_tigo,
        "PT - Portabilidad",
        [
          'category' => 'Portabilidad - Confirmación ',
          'event' => "PT - Mensaje",
          'label' => $error,
          'msisdn' => $this->msisdn_tigo,
          'msisdn_other' => $this->msisdn_other,
        ]
          );
      }
    }
    if ($this->return) {
      if ($this->return['error']) {
        $response = $segment->tracking_data_segment_generic(
        $this->msisdn_tigo,
        "PT - Portabilidad",
        [
          'category' => 'Portabilidad - Confirmación ',
          'event' => "PT - Mensaje",
          'label' => $this->return['error'],
          'msisdn' => $this->msisdn_tigo,
          'msisdn_other' => $this->msisdn_other,
        ]
          );
      }
      elseif ($this->return['send']) {
        $response = $segment->tracking_data_segment_generic(
        isset($this->msisdn_tigo)
        ? $this->msisdn_tigo
        : $this->number,
        "PT - Portabilidad",
        [
          'category' => 'Portabilidad - NIP ',
          'event' => "PT - Enviar NIP",
          'label' => 'Código Enviado',
          'msisdn' => isset($this->msisdn_tigo)
          ? $this->msisdn_tigo
          : $this->number,
          'msisdn_other' => isset($this->msisdn_other)
          ? $this->msisdn_other
          : FALSE,
        ]
        );
      }
    }
  }

}
