<?php

namespace Drupal\tigo_portability\tests\Unit\Controller;

use Drupal\Tests\UnitTestCase;
use Drupal\tigo_portability\Controller\TokenController;

/**
 * Class TokenController.
 *
 * @coversDefaultClass Drupal\tigo_portability\Controller\TokenController
 * @group tigo_portability
 */
class TokenControllerTest extends UnitTestCase {

  protected $object;
  protected $response;

  /**
   * {@inheritdoc}
   */
  public function setUp($tokenMock = NULL) {
    if ($tokenMock == NULL) {
      $tokenMock = $this->getCommonTokenMock('Drupal\tigo_portability\Services\Token');
    }
    $this->object = new TokenController($tokenMock);
  }

  /**
   * {@inheritdoc}
   */
  public function tearDown() {
    unset($this->object);
    unset($this->response);
  }

  /**
   * {@inheritdoc}
   */
  public function createTestMock($mockClass = NULL, $method = []) {
    $mock = $this->getMockBuilder($mockClass)
      ->disableOriginalConstructor()
      ->setMethods($method)
      ->getMock();
    return $mock;
  }

  /**
   * {@inheritdoc}
   */
  public function getCommonTokenMock($execute = NULL) {
    $tokenMock = $this->createTestMock('Drupal\tigo_portability\Services\Token', ['getToken']);
    $tokenMock->expects($this->any())
      ->method('getToken')
      ->willReturn([
        'token' => 'something',
        'code' => 200,
        'message' => 'message',
      ]);
    return $tokenMock;
  }

  /**
   * {@inheritdoc}
   */
  public function getCommonTokenMockFailed($execute = NULL) {
    $tokenMock = $this->createTestMock('Drupal\tigo_portability\Services\Token', ['getToken']);
    $tokenMock->expects($this->any())
      ->method('getToken')
      ->willReturn([
        'token' => FALSE,
        'code' => 401,
        'message' => 'message',
      ]);
    return $tokenMock;
  }

  /**
   * {@inheritdoc}
   */
  public function testPrueba() {
    $this->setUp();
    $this->assertNotFalse(TRUE);
    $this->tearDown();
  }

  /**
   * Pruebas sobre la fucnión de acceso.
   */
  public function testDontHaveAccessHttpRefererNull() {
    $_SERVER['HTTP_REFERER'] = NULL;
    $this->setUp();
    $this->response = $this->object->haveAccess();
    $this->assertEquals('401', $this->response['code']);
    $this->tearDown();
  }

  /**
   * Pruebas sobre la fucnión de acceso.
   */
  public function testDontHaveAccessHttpRefererEmpty() {
    $_SERVER['HTTP_REFERER'] = '';
    $this->setUp();
    $this->response = $this->object->haveAccess();
    $this->assertEquals('401', $this->response['code']);
    $this->tearDown();
  }

  /**
   * Pruebas sobre la fucnión de acceso.
   */
  public function testDontHaveAccessHttpRefererUnset() {
    unset($_SERVER['HTTP_REFERER']);
    $this->setUp();
    $this->response = $this->object->haveAccess();
    $this->assertEquals('401', $this->response['code']);
    $this->tearDown();
  }

  /**
   * Pruebas sobre la fucnión de acceso.
   */
  public function testHaveAccessHttpRefererAnyHost() {
    $_SERVER['HTTP_REFERER'] = 'any';
    $this->setUp();
    $this->response = $this->object->haveAccess();
    $this->assertEquals('200', $this->response['code']);
    $this->tearDown();
  }

  /**
   * Pruebas cuando el HOST es desde un servidor local.
   *
   * @depends testHaveAccessHttpRefererAnyHost
   */
  public function testGetTokenWithAccess() {
    $this->setUp($this->getCommonTokenMock('Drupal\tigo_portability\Services\Token'));
    $this->response = $this->object->getToken();
    $this->assertEquals(200, $this->response->getStatusCode());
    $this->tearDown();
  }

  /**
   * Pruebas cuando el HOST es desde un servidor local.
   *
   * @depends testDontHaveAccessHttpRefererUnset
   */
  public function testGetTokenWithNoAccess() {
    unset($_SERVER['HTTP_REFERER']);
    $this->setUp($this->getCommonTokenMockFailed('Drupal\tigo_portability\Services\Token'));
    $this->response = $this->object->getToken();
    $this->assertEquals(401, $this->response->getStatusCode());
    $this->tearDown();
  }

}
