<?php

namespace Drupal\tigo_portability\tests\Unit\Model;

use Drupal\Tests\UnitTestCase;
use Drupal\tigo_portability\Model\TokenModel;

/**
 * Class TokenModel.
 *
 * @coversDefaultClass Drupal\tigo_portability\Model\TokenModel
 * @group tigo_portability
 */
class TokenModelTest extends UnitTestCase {

  protected $object;
  protected $response;

  /**
   * {@inheritdoc}
   */
  public function setUp($connectionMock = NULL) {
    if ($connectionMock == NULL) {
      $executeMock = $this->createTestMock('executeMock', [
        'fetchAll',
      ]);
      $executeMock->expects($this->any())->method('fetchAll')->willReturn([]);
      $selectMock = $this->createTestMock('selectMock', [
        'fields',
        'condition',
        'execute',
      ]);
      $selectMock->expects($this->any())->method('execute')->willReturn($executeMock);
      $insertMock = $this->createTestMock('insertMock', [
        'fields',
        'execute',
      ]);
      $insertMock->expects($this->any())->method('execute')->willReturn(1);
      $connectionMock = $this->createTestMock('Drupal\Core\Database\Connection', [
        'select',
        'insert',
        'queryRange',
        'queryTemporary',
        'driver',
        'databaseType',
        'createDatabase',
        'mapConditionOperator',
        'nextId',
      ]);
      $connectionMock->expects($this->any())->method('select')->willReturn($selectMock);
      $connectionMock->expects($this->any())->method('insert')->willReturn($insertMock);
      $connectionMock->expects($this->any())->method('queryRange')->willReturn(NULL);
      $connectionMock->expects($this->any())->method('queryTemporary')->willReturn(NULL);
      $connectionMock->expects($this->any())->method('driver')->willReturn(NULL);
      $connectionMock->expects($this->any())->method('databaseType')->willReturn(NULL);
      $connectionMock->expects($this->any())->method('createDatabase')->willReturn(NULL);
      $connectionMock->expects($this->any())->method('mapConditionOperator')->willReturn(NULL);
      $connectionMock->expects($this->any())->method('nextId')->willReturn(NULL);
    }
    $this->object = new TokenModel($connectionMock);
  }

  /**
   * {@inheritdoc}
   */
  public function tearDown() {
    unset($this->object);
    unset($this->response);
  }

  /**
   * {@inheritdoc}
   */
  public function createTestMock($mockClass = NULL, $method = []) {
    $mock = $this->getMockBuilder($mockClass)
      ->disableOriginalConstructor()
      ->setMethods($method)
      ->getMock();
    return $mock;
  }

  /**
   * {@inheritdoc}
   */
  public function getCommonDrupalMock($execute = NULL) {
    $selectMock = $this->createTestMock('selectMock', ['select']);
    $selectMock->expects($this->any())->method('select')->willReturn('');
    $drupalMock = $this->createTestMock('drupalMock', ['database']);
    $drupalMock->expects($this->any())
      ->method('database')
      ->willReturn($selectMock);
    return $drupalMock;
  }

  /**
   * {@inheritdoc}
   */
  public function testPrueba() {
    $this->setUp();
    $this->assertNotFalse(TRUE);
    $this->tearDown();
  }

  /**
   * {@inheritdoc}
   */
  public function testGetMock() {
    $this->setUp();
    $this->response = $this->object->getToken('123456');
    $this->assertInternalType('array', $this->response);
    $this->tearDown();
  }

  /**
   * {@inheritdoc}
   */
  public function testSetToken() {
    $this->setUp();
    $this->response = $this->object->setToken([]);
    $this->assertInternalType('int', $this->response);
    $this->tearDown();
  }

  /**
   * {@inheritdoc}
   */
  public function testSetNipRequest() {
    $this->setUp();
    $this->response = $this->object->setNipRequest([]);
    $this->assertInternalType('int', $this->response);
    $this->tearDown();
  }

}
