<?php

namespace Drupal\tigo_portability\tests\Unit\Services;

use Drupal\Tests\UnitTestCase;
use Drupal\tigo_portability\Services\Token;

/**
 * Class Token.
 *
 * @coversDefaultClass Drupal\tigo_portability\Services\Token
 * @group tigo_portability
 */
class TokenTest extends UnitTestCase {

  protected $object;
  protected $response;

  /**
   * {@inheritdoc}
   */
  public function setUp(
    $token = TRUE,
    $connectionMock = NULL,
    $config_factory = NULL,
    $tokenModel = NULL
  ) {
    if ($connectionMock == NULL) {
      $connectionMock = $this->createTestMock('Drupal\Core\Database\Connection');
    }
    if ($config_factory == NULL) {
      // Source: https://drupal.stackexchange.com/questions/250868/injecting-configuration-into-mock-config-object.
      $config_factory = $this->getConfigFactoryStub([
        'tigo_portability.adminsettings' => ['time_available' => 1],
      ]);
    }
    if ($tokenModel == NULL) {
      $tokenModel = $this->createTestMock('Drupal\tigo_portability\Model\TokenModel', [
        'setToken',
        'getToken',
      ]);
      $tokenModel->expects($this->any())->method('setToken')->willReturn(1);
      if ($token) {
        $tokenModel->expects($this->any())->method('getToken')->willReturn([
          'id' => 1,
          'token' => 'any',
          'created' => '2019-01-22 08:34:00',
          'expired' => '2019-04-15 17:35:00',
          'domain' => 'http://transaccionales-co.local/admin/config/portabilidad/adminsettings',
        ]);
      }
      else {
        $tokenModel->expects($this->any())->method('getToken')->willReturn([]);
      }
    }
    $this->object = new Token($connectionMock, $config_factory, $tokenModel);
  }

  /**
   * {@inheritdoc}
   */
  public function tearDown() {
    unset($this->object);
    unset($this->response);
  }

  /**
   * {@inheritdoc}
   */
  public function createTestMock($mockClass = NULL, $method = []) {
    $mock = $this->getMockBuilder($mockClass)
      ->disableOriginalConstructor()
      ->setMethods($method)
      ->getMock();
    return $mock;
  }

  /**
   * {@inheritdoc}
   */
  public function testPrueba() {
    $this->setUp();
    $this->assertNotFalse(TRUE);
    $this->tearDown();
  }

  /**
   *
   */
  public function testGenerateTokenFromLocalHost() {
    $this->setUp();
    $this->response = $this->object->generateToken();
    $this->assertInternalType('string', $this->response);
    $this->tearDown();
  }

  /**
   *
   */
  public function testGenerateTokenFromRemoteHost() {
    $_SERVER['HTTP_REFERER'] = 'any';
    $this->setUp();
    $this->response = $this->object->generateToken();
    $this->assertInternalType('string', $this->response);
    $this->tearDown();
  }

  /**
   *
   */
  public function testTokenExist() {
    $this->setUp();
    $this->response = $this->object->tokenExist('any');
    $this->assertInternalType('array', $this->response);
    $this->tearDown();
  }

  /**
   *
   */
  public function testTokenNotExist() {
    $this->setUp(FALSE);
    $this->response = $this->object->tokenExist('any');
    $this->assertFalse($this->response);
    $this->tearDown();
  }

  /**
   *
   */
  public function testValidateExpirationNotExpired() {
    $this->setUp();
    $expired = time() + 1800;
    $exist = [
      0 => (object) [
        'expired' => date('Y-m-d H:i:s', $expired),
      ],
    ];
    $this->response = $this->object->validateExpiration($exist);
    $this->assertNotFalse($this->response);
    $this->tearDown();
  }

  /**
   *
   */
  public function testValidateExpirationExpired() {
    $this->setUp();
    $exist = [
      0 => (object) [
        'expired' => '2019-01-22 08:34:00',
      ],
    ];
    $this->response = $this->object->validateExpiration($exist);
    $this->assertFalse($this->response);
    $this->tearDown();
  }

  /**
   *
   */
  public function testValidateExpirationExpiredNotSet() {
    $this->setUp();
    $this->response = $this->object->validateExpiration([]);
    $this->assertFalse($this->response);
    $this->tearDown();
  }

  /**
   *
   */
  public function testValidateToken() {
    $this->setUp();
    $this->response = $this->object->validateToken([]);
    $this->assertInternalType('bool', $this->response);
    $this->tearDown();
  }

  /**
   *
   */
  public function testValidateTokenEmpty() {
    $this->setUp(FALSE);
    $this->response = $this->object->validateToken([]);
    $this->assertInternalType('bool', $this->response);
    $this->tearDown();
  }

  /**
   * @depends testGenerateTokenFromRemoteHost
   */
  public function testGetToken() {
    $this->setUp();
    $this->response = $this->object->getToken();
    $this->assertInternalType('array', $this->response);
    $this->tearDown();
  }

}
